part of barcodes;

/// Renders the barcode control
/// ```dart
/// Widget build(BuildContext context) {
///    return Container(
///        child: SfBarcodeGenerator(value:'123456'));
///}
/// ```dart
class SfBarcodeGenerator extends StatefulWidget {
  /// Creates the SfBarcodeGenerator
  // ignore: prefer_const_constructors_in_immutables
  SfBarcodeGenerator(
      {Key key,
      @required this.value,
      Symbology symbology,
      this.barColor,
      this.backgroundColor,
      this.showValue = false,
      this.textSpacing = 2,
      this.textAlign = TextAlign.center,
      this.textStyle = const TextStyle()})
      : symbology = symbology ?? Code128(),
        super(key: key);

  /// Defines the value of the barcode to be rendered.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456'));
  ///}
  /// ```dart
  final String value;

  /// Define anyone of barcode symbology that will be used to convert
  /// input value into visual barcode representation.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        symbology: UPCE()));
  ///}
  /// ```dart
  final Symbology symbology;

  /// Define the color for barcode elements.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        barColor : Colors.red));
  ///}
  /// ```dart
  final Color barColor;

  /// Define the background color for the barcode area.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        backgroundColor : Colors.red));
  ///}
  /// ```dart
  final Color backgroundColor;

  /// Specifies whether to show the value along with the barcode.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        showValue : true));
  ///}
  /// ```dart
  final bool showValue;

  /// Specifies the spacing between the text and the barcode.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        textSpacing: 10));
  ///}
  /// ```dart
  final double textSpacing;

  /// Defines the text alignment for the text to be rendered in the barcode.
  /// The text alignment types are:
  //
  //left
  //
  //center
  //
  //right
  /// Specifies the spacing between the text and the barcode.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        textAlign: TextAlign.right));
  ///}
  /// ```dart
  final TextAlign textAlign;

  ///Add style to customize the text.
  /// ```dart
  /// Widget build(BuildContext context) {
  ///    return Container(
  ///        child: SfBarcodeGenerator(value:'123456',
  ///        textStyle: TextStyle(fontSize: 16));
  ///}
  /// ```dart
  final TextStyle textStyle;

  @override
  State<StatefulWidget> createState() {
    return _SfBarcodeGeneratorState();
  }
}

/// Represents the barcode generator state
class _SfBarcodeGeneratorState extends State<SfBarcodeGenerator> {
  SfBarcodeThemeData _barcodeTheme;
  Size _textSize;
  @override
  Widget build(BuildContext context) {
    SyncfusionLicense.validateLicense(context);

    if (widget.showValue && _textSize == null) {
      _textSize = _measureText(widget.value.toString(), widget.textStyle);
    }
    widget.symbology._validateInput(widget.value);
    widget.symbology._textSize = _textSize;
    return Container(
      color: widget.backgroundColor ?? _barcodeTheme.backgroundColor,
      child: _SfBarcodeGeneratorRenderObjectWidget(
          value: widget.value,
          symbology: widget.symbology,
          foregroundColor: widget.barColor ?? _barcodeTheme.barColor,
          showText: widget.showValue,
          textSpacing: widget.textSpacing,
          textStyle: TextStyle(
              backgroundColor: widget.textStyle.backgroundColor,
              color: widget.textStyle.color ?? _barcodeTheme.textColor,
              fontSize: widget.textStyle.fontSize,
              fontFamily: widget.textStyle.fontFamily,
              fontStyle: widget.textStyle.fontStyle,
              fontWeight: widget.textStyle.fontWeight,
              textBaseline: widget.textStyle.textBaseline),
          textSize: _textSize,
          textAlign: widget.textAlign),
    );
  }

  @override
  void didChangeDependencies() {
    _barcodeTheme = SfBarcodeTheme.of(context);
    super.didChangeDependencies();
  }

  @override
  void didUpdateWidget(SfBarcodeGenerator oldWidget) {
    if (widget.showValue &&
        (oldWidget.value != widget.value ||
            oldWidget.textStyle != widget.textStyle)) {
      _textSize = _measureText(widget.value.toString(), widget.textStyle);
    }

    super.didUpdateWidget(oldWidget);
  }
}
