part of barcodes;

/// Represents the EAN13 symbology
class EAN13 extends Symbology {
  /// Creates the EAN13 symbology
  EAN13({int module})
      : super(
          module: module,
        );

  /// Represents the encoded input vale
  String _encodedValue;

  @override
  bool _validateInput(String value) {
    if (value.contains(RegExp(r'^(?=.*?[0-9]).{13}$')) &&
        int.parse(value[12]) == _checkSumData(value)) {
      _encodedValue = value;
    } else if (value.contains(RegExp(r'^(?=.*?[0-9]).{12}$'))) {
      _encodedValue = value + _checkSumData(value).toString();
    } else {
      throw 'EAN13 supports only numeric characters. The provided value should have'
          ' 12 digits (without checksum) or  with 13 digits';
    }
    return true;
  }

  /// Returns the encoded value
  List<String> _getCodeValues() {
    const String _endBar = '101';
    const String _middleBar = '01010';
    final Map<String, String> _structureValue = _getStructure();
    final String _structure =
        _structureValue.entries.elementAt(int.parse(_encodedValue[0])).value;
    final List<String> _code = <String>[];
    _code.add(_endBar);
    String _leftString = _encodedValue.substring(1, 7);
    _code.add(_getLeftValue(true, _structure, _leftString));
    _code.add(_middleBar);
    _leftString = _encodedValue.substring(7, 12);
    _code.add(_getLeftValue(false, 'RRRRRR', _leftString));
    _leftString = _encodedValue[12];
    _code.add(_getLeftValue(false, 'RRRRRR', _leftString));
    _code.add(_endBar);
    return _code;
  }

  /// To return the binary values of the supported input symbol
  Map<String, List<String>> _getBinaries() {
    return <String, List<String>>{
      'L': <String>[
        // The L (left) type of encoding
        '0001101', '0011001', '0010011', '0111101', '0100011',
        '0110001', '0101111', '0111011', '0110111', '0001011'
      ],
      'G': <String>[
        // The G type of encoding
        '0100111', '0110011', '0011011', '0100001', '0011101',
        '0111001', '0000101', '0010001', '0001001', '0010111'
      ],
      'R': <String>[
        // The R (right) type of encoding
        '1110010', '1100110', '1101100', '1000010', '1011100',
        '1001110', '1010000', '1000100', '1001000', '1110100'
      ],
      'O': <String>[
        // The O (odd) encoding for UPC-E
        '0001101', '0011001', '0010011', '0111101', '0100011',
        '0110001', '0101111', '0111011', '0110111', '0001011'
      ],
      'E': <String>[
        // The E (even) encoding for UPC-E
        '0100111', '0110011', '0011011', '0100001', '0011101',
        '0111001', '0000101', '0010001', '0001001', '0010111'
      ]
    };
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<String> code = _getCodeValues();
    final int _barTotalLength = _calculateTotalLength(code);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _barTotalLength, module, size.width, offset.dx);
    final Rect _barCodeRect = Rect.fromLTRB(
        offset.dx, offset.dy, offset.dx + size.width, offset.dy + size.height);
    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      final int singleModule = (size.width ~/ _barTotalLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_barTotalLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < code.length; i++) {
      final String _codeValue = code[i];
      const bool _hasExtraHeight = false;
      final double _barHeight = _hasExtraHeight
          ? size.height + _textSize.height + textSpacing
          : size.height;
      final int _codeLength = _codeValue.length;
      for (int j = 0; j < _codeLength; j++) {
        final bool _canDraw = _codeValue[j] == '1' ? true : false;
        if (_canDraw &&
            (_left >= _barCodeRect.left &&
                _left + _ratio < _barCodeRect.right)) {
          final Rect _individualBarRect = Rect.fromLTRB(
              _left, offset.dy, _left + _ratio, offset.dy + _barHeight);
          canvas.drawRect(_individualBarRect, _paint);
        }
        _left += _ratio;
      }
    }
    if (showValue)
      _drawText(canvas, offset, size, value, textStyle, textSpacing, textAlign);
  }

  /// Calculate total bar length from give input value
  int _calculateTotalLength(List<String> code) {
    int _count = 0;
    for (int i = 0; i < code.length; i++) {
      final int _numberOfDigits = code[i].length;
      _count += _numberOfDigits;
    }
    return _count;
  }

  /// Represents the structure of the supported input symbol
  Map<String, String> _getStructure() {
    return <String, String>{
      '0': 'LLLLLL',
      '1': 'LLGLGG',
      '2': 'LLGGLG',
      '3': 'LLGGGL',
      '4': 'LGLLGG',
      '5': 'LGGLLG',
      '6': 'LGGGLL',
      '7': 'LGLGLG',
      '8': 'LGLGGL',
      '9': 'LGGLGL'
    };
  }

  /// Method to calculate the check sum digit
  int _checkSumData(String _value) {
    final int _sum1 = 3 *
        (int.parse(_value[11]) +
            int.parse(_value[9]) +
            int.parse(_value[7]) +
            int.parse(_value[5]) +
            int.parse(_value[3]) +
            int.parse(_value[1]));
    final int _sum2 = int.parse(_value[10]) +
        int.parse(_value[8]) +
        int.parse(_value[6]) +
        int.parse(_value[4]) +
        int.parse(_value[2]) +
        int.parse(_value[0]);
    final int _checkSumValue = _sum1 + _sum2;
    final int _checkSumDigit = (10 - _checkSumValue) % 10;
    return _checkSumDigit;
  }

  /// Method to calculate the left value
  String _getLeftValue(bool _isLeft, String _structure, String _leftString) {
    String _code;
    List<String> _tempCodes;
    final Map<String, List<String>> _codes = _getBinaries();
    for (int i = 0; i < _leftString.length; i++) {
      if (_structure[i] == 'L') {
        _tempCodes = _codes.entries.elementAt(0).value;
      } else if (_structure[i] == 'G') {
        _tempCodes = _codes.entries.elementAt(1).value;
      } else if (_structure[i] == 'R') {
        _tempCodes = _codes.entries.elementAt(2).value;
      } else if (_structure[i] == 'O') {
        _tempCodes = _codes.entries.elementAt(3).value;
      } else if (_structure[i] == 'E') {
        _tempCodes = _codes.entries.elementAt(4).value;
      }

      final int _currentValue = int.parse(_leftString[i]);
      if (i == 0) {
        _code = _tempCodes[_currentValue];
      } else {
        _code += _tempCodes[_currentValue];
      }
    }
    return _code;
  }
}
