part of barcodes;

/// Represents the EAN-8 symbology
class EAN8 extends Symbology {
  /// Creates the EAN-8 symbology
  EAN8({int module})
      : super(
          module: module,
        );

  /// Represents the encoded input value
  String _encodedValue;

  @override
  bool _validateInput(String value) {
    if (value.contains(RegExp(r'^(?=.*?[0-9]).{8}$')) &&
        int.parse(value[7]) == _checkSumData(value)) {
      _encodedValue = value;
    } else if (value.contains(RegExp(r'^(?=.*?[0-9]).{7}$'))) {
      _encodedValue = value + _checkSumData(value).toString();
    } else {
      throw 'EAN8 supports only numeric characters. The provided value should have'
          ' 7 digits (without checksum) or  with 8 digits';
    }
    return true;
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<String> code = _getCodeValues();
    final int _barTotalLength = _calculateTotalLength(code);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _barTotalLength, module, size.width, offset.dx);
    final Rect _barCodeRect = Rect.fromLTRB(
        offset.dx, offset.dy, offset.dx + size.width, offset.dy + size.height);

    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      final int singleModule = (size.width ~/ _barTotalLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_barTotalLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < code.length; i++) {
      final String _codeValue = code[i];
      const bool _hasExtraHeight = false;
      final double _barHeight = _hasExtraHeight
          ? size.height + _textSize.height + textSpacing
          : size.height;
      final int _codeLength = _codeValue.length;
      for (int j = 0; j < _codeLength; j++) {
        final bool _canDraw = _codeValue[j] == '1' ? true : false;
        if (_canDraw &&
            (_left >= _barCodeRect.left &&
                _left + _ratio < _barCodeRect.right)) {
          final Rect _individualBarRect = Rect.fromLTRB(
              _left, offset.dy, _left + _ratio, offset.dy + _barHeight);
          canvas.drawRect(_individualBarRect, _paint);
        }
        _left += _ratio;
      }
    }
    if (showValue)
      _drawText(canvas, offset, size, value, textStyle, textSpacing, textAlign);
  }

  /// Calculate total bar length from given input value
  int _calculateTotalLength(List<String> code) {
    int _count = 0;
    for (int i = 0; i < code.length; i++) {
      final int _numberOfDigits = code[i].length;
      _count += _numberOfDigits;
    }
    return _count;
  }

  /// Returns the encoded value
  List<String> _getCodeValues() {
    const String _endBars = '101';
    const String _middleBar = '01010';
    Map<String, String> _codes = _getCodeValueRight(true);
    final List<String> _code = <String>[];
    _code.add(_endBars);
    _code.add(_getLeftValue(_codes, true));
    _code.add(_middleBar);
    _codes = _getCodeValueRight(false);
    _code.add(_getLeftValue(_codes, false));
    _code.add(_endBars);
    return _code;
  }

  /// Represents the encoded value for the first 6 digits of the input value
  String _getLeftValue(Map<String, String> _codes, bool _isLeft) {
    String _code = '';
    for (int i = _isLeft ? 0 : _encodedValue.length - 4;
        i < (_isLeft ? _encodedValue.length - 4 : _encodedValue.length);
        i++) {
      final int _currentValue = int.parse(_encodedValue[i]);
      if (i == 0 || i == 4) {
        _code = _codes.entries.elementAt(_currentValue).value;
      } else {
        _code += _codes.entries.elementAt(_currentValue).value;
      }
    }
    return _code;
  }

  /// Method to calculate the input data
  int _checkSumData(String _value) {
    for (int i = 0; i < _value.length; i++) {
      final int _sum1 =
          int.parse(_value[1]) + int.parse(_value[3]) + int.parse(_value[5]);
      final int _sum2 = 3 *
          (int.parse(_value[0]) +
              int.parse(_value[2]) +
              int.parse(_value[4]) +
              int.parse(_value[6]));
      final int _checkSumValue = _sum1 + _sum2;
      final int _checkSumDigit = (10 - _checkSumValue) % 10;
      return _checkSumDigit;
    }
    return 0;
  }

  /// Represents the encoded value for the last 6 digits of the input value
  Map<String, String> _getCodeValueRight(bool _isRight) {
    Map<String, String> _codes;
    if (_isRight) {
      _codes = <String, String>{
        '0': '0001101',
        '1': '0011001',
        '2': '0010011',
        '3': '0111101',
        '4': '0100011',
        '5': '0110001',
        '6': '0101111',
        '7': '0111011',
        '8': '0110111',
        '9': '0001011',
      };
    } else {
      _codes = <String, String>{
        '0': '1110010',
        '1': '1100110',
        '2': '1101100',
        '3': '1000010',
        '4': '1011100',
        '5': '1001110',
        '6': '1010000',
        '7': '1000100',
        '8': '1001000',
        '9': '1110100'
      };
    }
    return _codes;
  }
}
