part of barcodes;

/// Represents the UPCA codebar
class UPCA extends Symbology {
  /// Creates the UPCA code bar
  UPCA({int module})
      : super(
          module: module,
        );

  /// Represents the encoded input value
  String _encodedValue;

  @override
  bool _validateInput(String value) {
    if (value.contains(RegExp(r'^(?=.*?[0-9]).{11}$'))) {
      _encodedValue = value + _checkSumData(value).toString();
    } else if (value.contains(RegExp(r'^(?=.*?[0-9]).{12}$')) &&
        int.parse(value[11]) == _checkSumData(value)) {
      _encodedValue = value;
    } else {
      throw 'UPCA supports only numeric characters. The provided value should have'
          ' 11 digits (without checksum) or  with 12 digits';
    }
    return true;
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<String> code = _getCodeValues();
    final int _barTotalLength = _calculateTotalLength(code);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _barTotalLength, module, size.width, offset.dx);
    final Rect _barCodeRect = Rect.fromLTRB(
        offset.dx, offset.dy, offset.dx + size.width, offset.dy + size.height);
    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      final int singleModule = (size.width ~/ _barTotalLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_barTotalLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < code.length; i++) {
      final String _codeValue = code[i];
      const bool _hasExtraHeight = false;
      final double _barHeight = _hasExtraHeight
          ? size.height + _textSize.height + textSpacing
          : size.height;
      final int _codeLength = _codeValue.length;
      for (int j = 0; j < _codeLength; j++) {
        final bool _canDraw = _codeValue[j] == '1' ? true : false;
        if (_canDraw &&
            (_left >= _barCodeRect.left &&
                _left + _ratio < _barCodeRect.right)) {
          final Rect _individualBarRect = Rect.fromLTRB(
              _left, offset.dy, _left + _ratio, offset.dy + _barHeight);
          canvas.drawRect(_individualBarRect, _paint);
        }
        _left += _ratio;
      }
    }
    if (showValue)
      _drawText(canvas, offset, size, value, textStyle, textSpacing, textAlign);
  }

  /// Calculate total bar length from give input value
  int _calculateTotalLength(List<String> code) {
    int _count = 0;
    for (int i = 0; i < code.length; i++) {
      final int _numberOfDigits = code[i].length;
      _count += _numberOfDigits;
    }
    return _count;
  }

  /// Returns the encoded value
  List<String> _getCodeValues() {
    const String _endDigits = '00000000';
    const String _middleBar = '01010';
    final List<String> _code = <String>[];
    _code.add(_endDigits);
    _code.add('101' + _getLeftValue(true, 'L', _encodedValue[0]));
    _code.add(_getLeftValue(true, 'LLLLL', _encodedValue.substring(1, 6)));
    _code.add(_middleBar);
    _code.add(_getLeftValue(true, 'RRRRR', _encodedValue.substring(6, 11)));
    _code.add(_getLeftValue(true, 'R', _encodedValue[11]) + '101');
    _code.add(_endDigits);
    return _code;
  }

  /// Returns the binary values for the supported input symbol
  Map<String, List<String>> _getBinaries() {
    final Map<String, List<String>> _codes = <String, List<String>>{
      'L': <String>[
        '0001101',
        '0011001',
        '0010011',
        '0111101',
        '0100011',
        '0110001',
        '0101111',
        '0111011',
        '0110111',
        '0001011'
      ],
      'R': <String>[
        '1110010',
        '1100110',
        '1101100',
        '1000010',
        '1011100',
        '1001110',
        '1010000',
        '1000100',
        '1001000',
        '1110100'
      ]
    };

    return _codes;
  }

  /// Returns the encoded value of digits present at left side
  String _getLeftValue(bool _isLeft, String _structure, String _leftString) {
    String _code;
    List<String> _tempValue;
    final Map<String, List<String>> _codes = _getBinaries();
    for (int i = 0; i < _leftString.length; i++) {
      if (_structure[i] == 'R') {
        _tempValue = _codes.entries.elementAt(1).value;
      } else {
        _tempValue = _codes.entries.elementAt(0).value;
      }

      final int _currentValue = int.parse(_leftString[i]);
      if (i == 0) {
        _code = _tempValue[_currentValue];
      } else {
        _code += _tempValue[_currentValue];
      }
    }
    return _code;
  }

  /// Method to calculate the check sum digit
  int _checkSumData(String _value) {
    final int _sum1 = 3 *
        (int.parse(_value[0]) +
            int.parse(_value[2]) +
            int.parse(_value[4]) +
            int.parse(_value[6]) +
            int.parse(_value[8]) +
            int.parse(_value[10]));
    final int _sum2 = int.parse(_value[9]) +
        int.parse(_value[7]) +
        int.parse(_value[5]) +
        int.parse(_value[3]) +
        int.parse(_value[1]);
    final int _checkSumValue = _sum1 + _sum2;
    return (10 - _checkSumValue % 10) % 10;
  }
}
