part of barcodes;

/// Represents the UPCE
class UPCE extends Symbology {
  /// Creates the UPCEbar code
  UPCE({int module})
      : super(
          module: module,
        );

  @override
  bool _validateInput(String value) {
    if (value.contains(RegExp(r'^(?=.*?[0-9]).{6}$'))) {
      return true;
    }
    throw 'UPCE supports only numeric characters. The provided value should have'
        ' 6 digits';
  }

  @override
  void _renderBarcode(
      Canvas canvas,
      Size size,
      Offset offset,
      String value,
      Color foregroundColor,
      TextStyle textStyle,
      double textSpacing,
      TextAlign textAlign,
      bool showValue) {
    final Paint _paint = _getBarPaint(foregroundColor);
    final List<String> code = _getCodeValues(value);
    final int _barTotalLength = _calculateTotalLength(code);
    double _left = module == null
        ? offset.dx
        : _calculateLeftPosition(
            _barTotalLength, module, size.width, offset.dx);
    final Rect _barCodeRect = Rect.fromLTRB(
        offset.dx, offset.dy, offset.dx + size.width, offset.dy + size.height);
    double _ratio = 0;
    if (module != null) {
      _ratio = module.toDouble();
    } else {
      final int singleModule = (size.width ~/ _barTotalLength).toInt();
      _ratio = singleModule.toDouble();
      final double _leftPadding = (size.width - (_barTotalLength * _ratio)) / 2;
      _left += _leftPadding;
    }
    _left = _left.roundToDouble();
    for (int i = 0; i < code.length; i++) {
      final String _codeValue = code[i];
      const bool _hasExtraHeight = false;
      final double _barHeight = _hasExtraHeight
          ? size.height + _textSize.height + textSpacing
          : size.height;
      final int _codeLength = _codeValue.length;
      for (int j = 0; j < _codeLength; j++) {
        final bool _canDraw = _codeValue[j] == '1' ? true : false;
        if (_canDraw &&
            (_left >= _barCodeRect.left &&
                _left + _ratio < _barCodeRect.right)) {
          final Rect _individualBarRect = Rect.fromLTRB(
              _left, offset.dy, _left + _ratio, offset.dy + _barHeight);
          canvas.drawRect(_individualBarRect, _paint);
        }
        _left += _ratio;
      }
    }
    if (showValue)
      _drawText(canvas, offset, size, value, textStyle, textSpacing, textAlign);
  }

  /// Calculate the total length from given value
  int _calculateTotalLength(List<String> code) {
    int _count = 0;
    for (int i = 0; i < code.length; i++) {
      final int _numberOfDigits = code[i].length;
      _count += _numberOfDigits;
    }
    return _count;
  }

  /// Method to calculate the check sum value
  num _checkSum(String _value) {
    num _result = 0;
    for (int i = 1; i < 11; i += 2) {
      _result += int.parse(_value[i]);
    }
    for (int i = 0; i < 11; i += 2) {
      _result += int.parse(_value[i]) * 3;
    }
    return (10 - (_result % 10)) % 10;
  }

  /// Returns the supported symbol and its pattern value
  Map<String, String> _getStructure() {
    final Map<String, String> _upceSymbology = <String, String>{
      '0': 'EEEOOO',
      '1': 'EEOEOO',
      '2': 'EEOOEO',
      '3': 'EEOOOE',
      '4': 'EOEEOO',
      '5': 'EOOEEO',
      '6': 'EOOOEE',
      '7': 'EOEOEO',
      '8': 'EOEOOE',
      '9': 'EOOEOE'
    };
    return _upceSymbology;
  }

  /// Returns the byte value for the supported symbol
  List<String> _getValue() {
    return <String>[
      'XX00000XXX',
      'XX10000XXX',
      'XX20000XXX',
      'XXX00000XX',
      'XXXX00000X',
      'XXXXX00005',
      'XXXXX00006',
      'XXXXX00007',
      'XXXXX00008',
      'XXXXX00009'
    ];
  }

  /// Returns the value for the last digit
  String _getExpansion(String _lastDigit) {
    final List<String> _value = _getValue();
    final int _index = int.parse(_lastDigit);
    return _value[_index];
  }

  /// Returns the calculated UPC value
  String _getUPCValue(String value) {
    final String _lastDigit = value[value.length - 1];
    final String _expansionValue = _getExpansion(_lastDigit);
    String _result = '';
    num _index = 0;
    for (int i = 0; i < _expansionValue.length; i++) {
      final String _value = _expansionValue[i];
      if (_value == 'X') {
        _result += value[_index++];
      } else {
        _result += _value;
      }
    }
    _result = '0' + _result;
    String _encodingValue = '' + _result;

    _encodingValue += _checkSum(_result).toString();

    return _encodingValue;
  }

  /// Returns the binary values of the supported symbol
  Map<String, List<String>> _getBinaries() {
    return <String, List<String>>{
      'O': <String>[
        '0001101',
        '0011001',
        '0010011',
        '0111101',
        '0100011',
        '0110001',
        '0101111',
        '0111011',
        '0110111',
        '0001011'
      ],
      'E': <String>[
        // The E (even) encoding for UPC-E
        '0100111', '0110011', '0011011', '0100001', '0011101',
        '0111001', '0000101', '0010001', '0001001', '0010111'
      ]
    };
  }

  /// Returns the encoded input value
  String _encoding(String _upcEValue, String _value, String _structure) {
    String _code = '';
    List<String> _tempValue;
    final Map<String, List<String>> _codes = _getBinaries();
    for (int i = 0; i < _value.length; i++) {
      if (_structure[i] == 'E') {
        _tempValue = _codes.entries.elementAt(1).value;
      } else {
        _tempValue = _codes.entries.elementAt(0).value;
      }
      if (i == 0) {
        final int _index = int.parse(_value[i]);
        _code = _tempValue[_index];
      } else {
        final int _index = int.parse(_value[i]);
        _code += _tempValue[_index];
      }
    }
    return _code;
  }

  /// Returns the encoded value
  List<String> _getCodeValues(String value) {
    const String _endBars = '101';
    const String _middleBar = '010101';
    const String _endDigits = '00000000';
    final List<String> _code = <String>[];
    final String _upcEValue = _getUPCValue(value);
    final Map<String, String> _structureValue = _getStructure();
    final int _value = int.parse(_upcEValue[_upcEValue.length - 1]);
    final String _structure = _structureValue.entries.elementAt(_value).value;
    _code.add(_endDigits);
    _code.add(_endBars);
    _code.add(_encoding(_upcEValue, value, _structure));
    _code.add(_middleBar);
    _code.add(_endDigits);
    return _code;
  }
}
